#include "TimSynth.h"

#pragma udata renderchain_data = 0x500
func chain[32];

#pragma udata
static unsigned char cnt;
static unsigned char vol;
static unsigned char chain_cfg;

void audio_renderchain_init(unsigned char cfg, unsigned char volume)
{
	audio_renderchain_clear();
	audio_renderchain_set_config(cfg);
	audio_renderchain_set_volume(volume);
}

void audio_renderchain_set_volume(unsigned char volume)
{
	vol = volume;
}

void audio_renderchain_set_config(unsigned char cfg)
{
	chain_cfg = cfg;
}

void audio_renderchain_clear(void)
{
	for(cnt = 0; cnt < 32; cnt++)
	{
		chain[cnt] = 0;
	}
}

void audio_renderchain_put(unsigned char slot, func *cfunc)
{
	if(cfunc)
	{
		if(slot < 32)
		{
			chain[slot] = (func)cfunc;
		}
	}
}

void audio_renderchain_remove(unsigned char slot)
{
	if(slot < 32)
	{
		chain[slot] = 0;
	}
	DO_COREBUF = 0;
}

void audio_renderchain_run(void)
{
	if(chain_cfg & START_WITH_ADC)
	{	
		if(DO_COREBUF_1)
		{
			copy_buf8_to_buf16(adc_buf_1, downbuf);
		}
		else if(DO_COREBUF_2)
		{
			copy_buf8_to_buf16(adc_buf_2, downbuf);
		}
	}
	else
	{
		clear_buf16(downbuf);
	}

	for(cnt = 0; cnt < 32; cnt++)
	{
		if(chain[cnt])
		{
			(chain[cnt])();
		}
	}

	if(chain_cfg & FINAL_VOLUME_ADJUST)
	{	
		buf16_volume(downbuf, vol);
	}
	
	if(chain_cfg & FINAL_TO_DAC)
	{	
		if(DO_COREBUF_1)
		{
			copy_buf16_to_buf8(downbuf, dac_buf_1);
		}
		else if(DO_COREBUF_2)
		{
			copy_buf16_to_buf8(downbuf, dac_buf_2);
		}
	}
	
	DO_COREBUF = 0;
}
