#include "TimSynth.h"

#pragma udata access audiodata
near unsigned char adc_sample;
near unsigned char dac_sample;
near unsigned char sample_num;
near unsigned char fsrh_save, fsrl_save;
near unsigned char noise;
near unsigned char noisecnt;

near unsigned char old_bled;
near unsigned short old_button;

#pragma udata

#pragma code irq_high_entry = 0x08
void irq_high_entry(void)
{
	_asm goto irq_high _endasm
}

#pragma code

void irq_init(void)
{
	sample_num = 0;
	
	old_bled = 0xFF;
	old_button = 0xFF;
	
	SEL1 = 1;
	
	noisecnt = 0;
	
	RCONbits.IPEN = 1;
	
	INTCONbits.GIEH = 1;
	INTCONbits.GIEL = 1;

	INTCON2bits.RBPU = 0;
	
	IPR1bits.ADIP = 0;
	IPR1bits.TMR2IP = 1;
	IPR1bits.SSPIP = 0;
	IPR2bits.TMR3IP = 0;
	INTCON2bits.TMR0IP = 0;

	SEL2 = 1;

	PIE1bits.ADIE = 1;
	PIE1bits.TMR2IE = 1;
	PIE1bits.SSPIE = 1;
	PIE2bits.TMR3IE = 1;
	INTCONbits.TMR0IE = 1;

	T2CONbits.TMR2ON = 1;
	T3CONbits.TMR3ON = 1;
}

/*
**	
**	Code for the HIGH priority interrupts.
**	This is where the audio sampling rate is determined.
**	It is tied to TMR2 for that purpose
**	That timer is then also used for the PWM peripherial
**	PWM runs in the backround, while the associated TMR2 has a
**	postscaler assigned, which is what this interrupt reacts to.
**	
**	No other interrupt may use this high priority IRQ
*/


#pragma code irq_high=0x0800
void irq_high(void)
{
	_asm
		BSF		ASM_SEL2
	
DO_TMR2:	
		BSF		ADCON0, 2, ACCESS		// select audio input ADC channel
		BSF		ADCON0, 1, ACCESS		// start conversion
		BCF		PIR1, 1, ACCESS			// clear TMR2 interrupt flag
		
		BCF		CCP2CON, 4, ACCESS		// prepare the update of the PWM output
		BCF		CCP2CON, 5, ACCESS		// by starting to clear the lower two bits

TMR2_CHECK_DAC_BIT0:
		RRCF	dac_sample, F, ACCESS
		BNC		TMR2_CHECK_DAC_BIT1
		BSF		CCP2CON, 4, ACCESS		// .. if not, set bit0 of pwm output register
		
TMR2_CHECK_DAC_BIT1:
		RRCF	dac_sample, W, ACCESS
		BNC		TMR2_UPDATE_PWM_REG
		BSF		CCP2CON, 5, ACCESS		// .. if not, set bit0 of pwm output register

TMR2_UPDATE_PWM_REG:
		ANDLW	0x3F					// mask away the upper bits in WREG
		MOVWF	CCPR2L, ACCESS			// move sample from WREG into PWM output buffer

TMR2_HANDLE_DAC_BUFFER:
		MOVFF	FSR2H, fsrh_save		// save indirect address
		MOVFF	FSR2L, fsrl_save
		
		MOVLW	SBUF_BASE				// load hi byte of indirect address
		MOVWF	FSR2H, ACCESS
		MOVFF	sample_num, FSR2L		// load low-byte of indirect address

		MOVFF	INDF2, dac_sample		// store in sample register
		BTG		dac_sample, 7, ACCESS	// convert dac sample to signed
		
TMR2_HANDLE_ADC_BUFFER:
		BTG		adc_sample, 7, ACCESS	// convert adc sample to signed
		BSF		FSR2L, 7, ACCESS		// point to ADC buffer
		MOVFF	adc_sample, INDF2		// move sample from ADC to ADC buffer
		
		// prepare sample counter
		INCF	sample_num, F, ACCESS

		MOVF	sample_num, W, ACCESS	// move sample number to WREG

TMR2_CHECK_64:
		SUBLW	0x40					// substract 64 from WREG
		BNZ		TMR2_CHECK128			// if not zero, check for next level
		BSF		core_buffer_flags, 0, ACCESS

TMR2_CHECK128:		
		SUBLW	0xC0					// substract 192 from WREG again
		BNZ		TMR2_CHECKS_DONE		// if not zero, checks done
		BSF		core_buffer_flags, 1, ACCESS

TMR2_CHECKS_DONE:
		BCF		sample_num, 7, ACCESS
		
TMR2_DO_NOISEBUFFER:		
		MOVLW	NOISEBUF_BASE
		MOVWF	FSR2H, ACCESS
		
		MOVF	noisecnt, W, ACCESS
		ANDLW	0x3F
		MOVWF	FSR2L, ACCESS
		
		INCF	noisecnt, F, ACCESS

TMR2_GOOD_NOISE:
		ADDWF	adc_sample, W, ACCESS
		ADDWF	INDF2, W, ACCESS
		ADDWF	dac_sample, W, ACCESS

TMR2_CHEAP_NOISE:
		ADDWF	noise, W, ACCESS
		
		MOVWF	INDF2, ACCESS
				
TMR2_DONE:		
		MOVFF	fsrh_save, FSR2H		// restore indirect address
		MOVFF	fsrl_save, FSR2L
		
		RETFIE	1
	_endasm
}

#pragma code




/*
**	
**	Code for the low priority interrupts.
**	Here we create the seqtiming timebase using TMR0
**	These handle the button and led i/o as well using TMR3
**	Auto-Update of the '595 register using SSP
**	Sample the noise from ADC0
**	
*/

#pragma code irq_low=0x18
#pragma interruptlow irq_low

void irq_low(void)
{
	_asm

	BSF		ASM_SEL2

// AD interrupt:
//
// Read a audio sample, sampling started by main audio IRQ (high priority)
// Once done reading the audio sample, it will auto-sample the noise ADC input

CHECK_ADIF:							// AD interrupt flag handling

	BTFSS	PIR1, 6, ACCESS			// Check ADC interrupt flag
	BRA		END_CHECK_ADIF			// .. If not set, skip the ADC code

	MOVF	ADRESH, W, ACCESS 		// AD was called, store result in WREG	
	BCF		PIR1, 6, ACCESS			// Clear the AD interrupt flags

	BTFSC	ADCON0, 2, ACCESS		// Check if the noise-channel (0) was selected
	MOVWF	adc_sample, ACCESS		// .. if not (1), it was ADC, so let's place it into the adc_sample register

	ADDWF	noise, F, ACCESS 		// add current readout to noise register anyways

	BTG		ADCON0, 2, ACCESS		// prepare to sample other channel
	BTFSS	ADCON0, 2, ACCESS		// if it would not be the adc channel (1) again
	BSF		ADCON0, 1, ACCESS		// .. it was (0), so start sampling of noise

END_CHECK_ADIF:						// end of A/D conversion routine



// TMR0 interrupt
//
// used as seqtiming timebase

CHECK_TMR0:
	BTFSS	INTCON, 0x2, ACCESS
	BRA		END_CHECK_TMR0		//CHECK_SSPIF // 0x168

	BCF		INTCON, 0x2, ACCESS

	// prepare TMR0 for next shot by adding the requiered number
	// ticks to the TMR0 register set
	MOVF	TMR0L, W, ACCESS			// copy current TMR0L to seq_t low byte
	MOVWF	seq_t, ACCESS

	MOVF	TMR0H, W, ACCESS			// copy current TMR0H to seq_t high byte
	MOVWF	seq_t+1, ACCESS

	MOVF	t0_reload, W, ACCESS		// move required ticks low byte to WREG
	ADDWF	seq_t, F, ACCESS			// add WREG to seq_t low byte, store result in seq_t low byte
	MOVF	t0_reload+1, W, ACCESS		// move required ticks high byte to WREG
	ADDWFC	seq_t+1, F, ACCESS			// add WREG to seq_t high byte, include carry from low byte add, store result in seq_t high byte

	MOVF	seq_t+1, W, ACCESS
	MOVWF	TMR0H, ACCESS
	MOVF	seq_t, W, ACCESS
	MOVWF	TMR0L, ACCESS

TMR0_CHECK_MICROSTEP:
	DECFSZ	microstep, F, ACCESS
	BRA		END_CHECK_TMR0

	CLRF	new_step, ACCESS
	BSF		microstep, 2, ACCESS
	BSF		new_step, 0, ACCESS

	// move step indicator by one step
	BCF		step_pos, 0x7, ACCESS
	RLNCF	step_pos, F, ACCESS
	BNZ		STEP_INDICATOR_DONE
	BSF		step_pos, 0, ACCESS
	
STEP_INDICATOR_DONE:
	INCF	current_step, F, ACCESS

	MOVF	num_steps, W, ACCESS
	CPFSEQ	current_step, ACCESS
	BRA		TMR0_STEPMAX_NOT_REACHED
	
TMR0_STEPMAX_REACHED:
	CLRF	current_step, ACCESS
	MOVLW	0x01
	MOVWF	step_pos, ACCESS

	BSF		new_step, 1, ACCESS

	// move step indicator by one step
	BCF		bar_pos, 0x7, ACCESS
	RLNCF	bar_pos, F, ACCESS
	BNZ		BAR_INDICATOR_DONE
	BSF		bar_pos, 0, ACCESS
	
BAR_INDICATOR_DONE:
	INCF	current_bar, F, ACCESS
	MOVF	num_bars, W, ACCESS
	CPFSEQ	current_bar, ACCESS
	BRA		TMR0_BARMAX_NOT_REACHED

TMR0_BARMAX_REACHED:
	CLRF	current_bar, ACCESS
	MOVLW	0x01
	MOVWF	bar_pos, ACCESS

TMR0_BARMAX_NOT_REACHED:
TMR0_STEPMAX_NOT_REACHED:
END_CHECK_TMR0:




// TMR3 interrupt:
//
// Used to scan the keys, generate keyclick-flags, etc.
// Also used to update the board's led's

CHECK_TMR3:		
	BTFSS	PIR2, 1, ACCESS				// Check if TMR3 needs to be handled
	BRA		END_CHECK_TMR3				// If not, skip the TMR3 code
	BCF		PIR2, 1, ACCESS				// Clear the TMR3 interrupt flag
	BSF		TMR3H, 6, ACCESS			// Set bit 6 in TMR3H counter register

	MOVF	leds, W, ACCESS				// move led register into WREG
	
	TSTFSZ	led_cnt, ACCESS				// check if we are at the first cycle, led_cnt = 0
	BRA		TMR3_DONE_FIRST_CYCLE		// if not zero, skip this section

	BSF		led_cnt, 0, ACCESS			// if zero, set bit0 in led_cnt

	
TMR3_CHECK_BUTTONS:
	MOVFF	button, old_button			// Save the current button readouts		
	MOVFF	button+1, old_button+1		

	CLRF	button, ACCESS				// clear the current button readouts
	CLRF	button+1, ACCESS

	BTFSS	ASM_B1						// Read each individual button
	BSF		button, 0, ACCESS
	BTFSS	ASM_B2
	BSF		button, 1, ACCESS
	BTFSS	ASM_B3
	BSF		button, 2, ACCESS
	BTFSS	ASM_B4
	BSF		button, 3, ACCESS
	BTFSS	ASM_B5
	BSF		button, 4, ACCESS
	BTFSS	ASM_B6
	BSF		button, 5, ACCESS
	BTFSS	ASM_B7
	BSF		button, 6, ACCESS
	BTFSS	ASM_B8
	BSF		button, 7, ACCESS
	BTFSS	ASM_BM
	BSF		button+1, 0, ACCESS
TMR3_DONE_CHECK_BUTTONS:
	
TMR3_GENERATE_BUTTON_CLICKS:
	// C-equiv: button_clicked.bits.raw = (old_button ^ button.bits.raw) & button.bits.raw;
	// C-equiv: button_released.bits.raw = old_button ^ button.bits.raw

	MOVF	old_button+1, W, ACCESS
	XORWF	button+1, W, ACCESS
	XORWF	button_released+1, F, ACCESS

	ANDWF	button+1, W, ACCESS
	MOVWF	button_clicked+1, ACCESS

	MOVF	old_button, W, ACCESS
	XORWF	button, W, ACCESS
	XORWF	button_released, F, ACCESS

	ANDWF	button, W, ACCESS
	MOVWF	button_clicked, ACCESS
	
	
TMR3_DONE_GENERATE_BUTTON_CLICKS:

	MOVF	button, W, ACCESS
	
TMR3_CHECK_BUTTON_LED:
	BTFSS	ASM_SHOW_BUTTON_LED		// check if we want to have button states in button led's
	MOVLW	0x00	
TMR3_DONE_CHECK_BUTTON_LED:
	
TMR3_CHECK_BRIGHTEST_LED:	
	BTFSC	ASM_SHOW_BRIGHT_LED			// check if we want to have button bright led
	IORWF	bright_led, W, ACCESS		// if we do, add them to the output led register
TMR3_DONE_CHECK_BRIGHTEST_LED:	


TMR3_DONE_FIRST_CYCLE:


TMR3_CHECK_MEDLED:
	BTFSS	led_cnt, 4, ACCESS
	BRA		TMR3_DONE_CHECK_MED_LED

	BTFSC	ASM_SHOW_MED_LED			// check if we want to have medium led
	IORWF	med_led, W, ACCESS			// if we are, add med led's to the output led register


	BTFSC	ASM_SHOW_STEP_MED			// check if we want to have dim step indicator
	IORWF	step_pos, W, ACCESS			// if we are, add step indicator the output led register
	BTFSC	ASM_SHOW_BAR_MED			// check if we want to have dim step indicator
	IORWF	bar_pos, W, ACCESS			// if we are, add step indicator the output led register
TMR3_DONE_CHECK_MED_LED:

TMR3_CHECK_LOWLED:
	BTFSS	led_cnt, 7, ACCESS
	BRA		TMR3_DONE_CHECK_LOW_LED

	BTFSC	ASM_SHOW_DIM_LED			// check if we want to have dim led
	IORWF	dim_led, W, ACCESS			// if we are, add med led's to the output led register

	BTFSC	ASM_SHOW_STEP_DIM			// check if we want to have dim step indicator
	IORWF	step_pos, W, ACCESS			// if we are, add step indicator the output led register
	BTFSC	ASM_SHOW_BAR_DIM			// check if we want to have dim step indicator
	IORWF	bar_pos, W, ACCESS			// if we are, add step indicator the output led register
TMR3_DONE_CHECK_LOW_LED:
	
TMR3_CHECK_CHANGED_LED:
	MOVWF	leds, ACCESS

	MOVF	old_bled, W, ACCESS
	SUBWF	leds, W, ACCESS
	BZ		TMR3_DONE_CHECK_CHANGED_LED

	MOVF	leds, W, ACCESS
	MOVWF	old_bled, ACCESS
	XORLW	0xFF
	BCF		ASM_SEL1
	MOVWF	SSPBUF, ACCESS
TMR3_DONE_CHECK_CHANGED_LED:

TMR3_ADVANCE_LEDCNT:
	RLNCF	led_cnt, F, ACCESS
	BCF		led_cnt, 0, ACCESS
END_CHECK_TMR3:




// done sending SPI?
CHECK_SSPIF:
	BTFSS	PIR1, 3, ACCESS
	BRA		END_CHECK_SSPIF		//EXIT_IRQ
	BCF		PIR1, 3, ACCESS
	BSF		ASM_SEL1
END_CHECK_SSPIF:
	
EXIT_IRQ:
_endasm
}

#pragma code





