#include "TimSynth.h"

#include "drumsynth.h"

struct _ds drum_data;

#pragma udata drumbuf = 0x400
signed short drumbuf[BUFSIZE];

#pragma udata voicebuf = 0x480
signed char voicebuf[BUFSIZE];

#pragma udata
static signed short *obuf;

void drumsynth_init(void)
{
	drum_data.trigger.trigger_state = 0;
	drum_data.play.play_state = 0x00;
	clear_buf16(drumbuf);

	drumsynth_set_sample(&drum_data.voice_data[0], &bassdrum, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[1], &lowtom, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[2], &midtom, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[3], &hitom, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[4], &rimshot, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[5], &snare, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[6], &openhat, 200, 0x00);
	drumsynth_set_sample(&drum_data.voice_data[7], &closedhat, 200, 0x40);
}

void drumsynth_trigger(unsigned char tr)
{
	drum_data.trigger.trigger_state = tr;
	
	if(IS_TRIGGER0)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[0].mutes;
		IS_PLAYING0 = 1;
		drumsynth_start_voice(&drum_data.voice_data[0]);
	}

	if(IS_TRIGGER1)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[1].mutes;
		IS_PLAYING1 = 1;
		drumsynth_start_voice(&drum_data.voice_data[1]);
	}

	if(IS_TRIGGER2)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[2].mutes;
		IS_PLAYING2 = 1;
		drumsynth_start_voice(&drum_data.voice_data[2]);
	}

	if(IS_TRIGGER3)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[3].mutes;
		IS_PLAYING3 = 1;
		drumsynth_start_voice(&drum_data.voice_data[3]);
	}

	if(IS_TRIGGER4)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[4].mutes;
		IS_PLAYING4 = 1;
		drumsynth_start_voice(&drum_data.voice_data[4]);
	}

	if(IS_TRIGGER5)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[5].mutes;
		IS_PLAYING5 = 1;
		drumsynth_start_voice(&drum_data.voice_data[5]);
	}

	if(IS_TRIGGER6)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[6].mutes;
		IS_PLAYING6 = 1;
		drumsynth_start_voice(&drum_data.voice_data[6]);
	}

	if(IS_TRIGGER7)
	{
		drum_data.play.play_state &= ~drum_data.voice_data[7].mutes;
		IS_PLAYING7 = 1;
		drumsynth_start_voice(&drum_data.voice_data[7]);
	}
}


void drumsynth_set_sample(struct _dv_data *voice_data, const rom struct _smpl *sample, unsigned char volume, unsigned char mutes)
{
	static struct _dv_data *dv_d;
	
	dv_d = voice_data;
	
	dv_d->samplestore = sample;
	dv_d->sample = dv_d->samplestore->sample_data;
	dv_d->numblocks = dv_d->samplestore->num_blocks;
	dv_d->blockcnt = 0;
	dv_d->voice_flags = dv_d->samplestore->sample_type;
	dv_d->volume = volume;
	dv_d->mutes = mutes;
}

void drumsynth_start_voice(struct _dv_data *voice_data)
{
	voice_data->sample = voice_data->samplestore->sample_data;
	voice_data->numblocks = voice_data->samplestore->num_blocks;
	voice_data->blockcnt = 0;
}


unsigned char drumsynth_render_voice(struct _dv_data *voice_data)
{
	static struct _dv_data *dv_d;
	
	dv_d = voice_data;
	
	if(dv_d->voice_flags & HALF_SR)
	{
		mix_rombuf8dup_to_buf16(dv_d->sample, obuf, dv_d->volume);
		dv_d->sample += BUFSIZE/2;
	}
	else
	{
		mix_rombuf8_to_buf16(dv_d->sample, obuf, dv_d->volume);
		dv_d->sample += BUFSIZE;
	}
				
	dv_d->blockcnt++;
	
	if(dv_d->blockcnt >= dv_d->numblocks)
	{
		return WAS_LAST_BLOCK;
	}
	else
	{
		return HAS_MORE_BLOCKS;
	}
}


void drumsynth_play(signed short *outbuf, unsigned char render_type)
{
	obuf = outbuf;
	
	if(render_type == RENDER_COPY)
	{
		clear_buf16(drumbuf);
	}
		
	if(IS_PLAYING0)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[0]))
		{
			IS_PLAYING0 = 0;
		}
	}

	if(IS_PLAYING1)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[1]))
		{
			IS_PLAYING1 = 0;
		}
	}

	if(IS_PLAYING2)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[2]))
		{
			IS_PLAYING2 = 0;
		}
	}

	if(IS_PLAYING3)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[3]))
		{
			IS_PLAYING3 = 0;
		}
	}

	if(IS_PLAYING4)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[4]))
		{
			IS_PLAYING4 = 0;
		}
	}

	if(IS_PLAYING5)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[5]))
		{
			IS_PLAYING5 = 0;
		}
	}

	if(IS_PLAYING6)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[6]))
		{
			IS_PLAYING6 = 0;
		}
	}

	if(IS_PLAYING7)
	{
		if(drumsynth_render_voice(&drum_data.voice_data[7]))
		{
			IS_PLAYING7 = 0;
		}
	}
}

