#ifndef DRUMSYNTH_H
#define DRUMSYNTH_H

#include "audio_core.h"
#include "samples.h"
#include "audio_buffers.h"

#define VOICE0	0x01
#define VOICE1	0x02
#define VOICE2	0x04
#define VOICE3	0x08
#define VOICE4	0x10
#define VOICE5	0x20
#define VOICE6	0x40
#define VOICE7	0x80

#define IS_PLAYING0	drum_data.play.play_0
#define IS_PLAYING1	drum_data.play.play_1
#define IS_PLAYING2	drum_data.play.play_2
#define IS_PLAYING3	drum_data.play.play_3
#define IS_PLAYING4	drum_data.play.play_4
#define IS_PLAYING5	drum_data.play.play_5
#define IS_PLAYING6	drum_data.play.play_6
#define IS_PLAYING7	drum_data.play.play_7

#define IS_TRIGGER0	drum_data.trigger.trigger_0
#define IS_TRIGGER1	drum_data.trigger.trigger_1
#define IS_TRIGGER2	drum_data.trigger.trigger_2
#define IS_TRIGGER3	drum_data.trigger.trigger_3
#define IS_TRIGGER4	drum_data.trigger.trigger_4
#define IS_TRIGGER5	drum_data.trigger.trigger_5
#define IS_TRIGGER6	drum_data.trigger.trigger_6
#define IS_TRIGGER7	drum_data.trigger.trigger_7

#define RENDER_COPY	0x01
#define RENDER_MIX	0x00

#define WAS_LAST_BLOCK	0x01
#define HAS_MORE_BLOCKS	0x00

typedef union _spos
{
	unsigned long short pos_cnt;
	struct
	{
		unsigned char pos_x;
		unsigned char pos_lo;
		unsigned char pos_hi;
	};
	struct
	{
		unsigned char pos_x1;
		unsigned short pos;
	};
};

typedef union _pstate
{
	unsigned char play_state;
	struct
	{
		unsigned char play_0:1;
		unsigned char play_1:1;
		unsigned char play_2:1;
		unsigned char play_3:1;
		unsigned char play_4:1;
		unsigned char play_5:1;
		unsigned char play_6:1;
		unsigned char play_7:1;
	};
};		

typedef union _tstate
{
	unsigned char trigger_state;
	struct
	{
		unsigned char trigger_0:1;
		unsigned char trigger_1:1;
		unsigned char trigger_2:1;
		unsigned char trigger_3:1;
		unsigned char trigger_4:1;
		unsigned char trigger_5:1;
		unsigned char trigger_6:1;
		unsigned char trigger_7:1;
	};
};		

typedef struct _dv_data
{
	rom struct _smpl *samplestore;
	rom signed char rom *sample;
	unsigned char numblocks;
	unsigned char blockcnt;
	unsigned char voice_flags;
	unsigned char mutes;
	unsigned char volume;
};	

typedef struct _ds
{
	union _tstate trigger;
	union _pstate play;

	struct _dv_data voice_data[8];
};

extern struct _ds drum_data;
extern signed short drumbuf[BUFSIZE];



void drumsynth_init(void);

void drumsynth_set_sample(struct _dv_data *voice_data, const rom struct _smpl *sample, unsigned char volume, unsigned char mutes);
void drumsynth_start_voice(struct _dv_data *voice_data);
unsigned char drumsynth_render_voice(struct _dv_data *voice_data);

void drumsynth_trigger(unsigned char tr);
void drumsynth_play(signed short *outbuf, unsigned char render_type);

#endif
